<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */

namespace App\Command;

use App\Helper\LoggerHelper;
use App\Jmcomic\Client;
use App\Jmcomic\ClientFactory;
use App\Model\Album;
use App\Model\Chapter;
use App\Model\ChapterImage;
use App\Transfer\ImageCenterDistributor;
use GuzzleHttp\Exception\GuzzleException;
use Hyperf\Command\Annotation\Command;
use Hyperf\Command\Command as HyperfCommand;
use Hyperf\Di\Annotation\Inject;
use Psr\Container\ContainerInterface;
use Symfony\Component\Console\Helper\ProgressBar;

/**
 * @Command
 */
#[Command]
class ComicListMonitorCommand extends HyperfCommand
{
    #[Inject]
    protected ClientFactory $clientFactory;

    #[Inject]
    protected ImageCenterDistributor $distributor;

    protected Client $client;

    public function __construct()
    {
        parent::__construct('comics:list:monitor');
    }

    public function configure()
    {
        parent::configure();
        $this->setDescription('18comic Korean comics list monitor');
    }

    public function handle()
    {
        $this->client = $this->clientFactory->create();
        // 检查有无新的漫画
        $page = 100000;
        while (true) {
            try {
                // print page
                $this->line("page: {$page}", 'info');
                $payload = $this->client->filter($page, 'hanman');
                var_dump($payload['total']);
                if (!isset($payload['content'])) {
                    $this->output->error('Failed to get content');
                    return;
                }
                foreach ($payload['content'] as $value) {
                    if (Album::query()->where('source_id', $value['id'])->exists()) {
                        continue;
                    }
                    $this->line("New album found: ${value['name']}", 'info');
                    $apiInfo = $this->client->album($value['id']);
                    $album = $this->enterAlbumData($apiInfo);
                    // 从web上获取上架时间跟更新时间
                    $webInfo = $this->client->albumWebInfo($value['id']);
                    $album->last_updated = $webInfo['updateDate'] ?? null;
                    $album->published = $webInfo['published'] ?? null;
                    $album->cover = $this->distributor->distribute([
                        'source_src' => "/media/albums/{$album->source_id}.jpg",
                        'source_type' => ImageCenterDistributor::SOURCE_TYPE_ALBUM_THUMB,
                        'from_album_id' => $album->id,
                    ]);
                    $album->save();
                    $this->line('Album saved');
                }
                ++$page;
            } catch (\Throwable|\Exception|GuzzleException $e) {
                LoggerHelper::printStackTrace($e);
            }
        }
    }

    // from response convert to album
    private function enterAlbumData($data): Album
    {
        $album = new Album();
        $album->source_id = $data['id'];
        $album->name = $data['name'];
        $album->description = $data['description'] ?? '';
//        $album->chapters = count($data['series']);
        $album->chapters = 0;
        $album->author = $data['author'];
        $album->tags = $data['tags'];
        $album->save();
        return $album;
    }


}
