<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */
namespace App\Helper;

use Hyperf\Contract\StdoutLoggerInterface;
use Hyperf\Utils\ApplicationContext;
use Throwable;

class LoggerHelper
{
    private static ?StdoutLoggerInterface $logger = null;

    public static function printStackTrace(Throwable $throwable): void
    {
        $logger = self::getLogger();
        $logger->error(sprintf('%s[%s] in %s', $throwable->getMessage(), $throwable->getLine(), $throwable->getFile()));
        $logger->error($throwable->getTraceAsString());
        if ($previous = $throwable->getPrevious()) {
            self::printStackTrace($previous);
        }
    }

    private static function getLogger()
    {
        if (self::$logger) {
            return self::$logger;
        }
        $container = ApplicationContext::getContainer();
        return self::$logger = $container->get(StdoutLoggerInterface::class);
    }
}
