<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */

namespace App\Jmcomic;

use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Hyperf\Contract\ConfigInterface;
use Hyperf\Di\Annotation\Inject;
use Hyperf\Guzzle\ClientFactory;

class APINodeObtain
{
    #[Inject]
    protected ConfigInterface $config;

    private Client $client;

    public function __construct(ClientFactory $clientFactory)
    {
        $this->client = $clientFactory->create();
    }

    /**
     * @throws Exception
     */
    public function get(): string
    {
        // 获取可用的节点
        $result = $this->checkNode($this->getNodes(), $this->getTimeout());
        if (empty($result)) {
            throw new Exception('no available node');
        }
        // find time min node
        $minTime = null;
        $minNode = '';
        foreach ($result as $item){
            if (is_null($minTime) || $item['time'] < $minTime){
                $minTime = $item['time'];
                $minNode = $item['node'];
            }
        }

        return $minNode;
    }

    private function getNodes(): array
    {
        return $this->config->get('nacos_config.jmcomic.api_nodes');
    }

    private function getTimeout(): float
    {
        return $this->config->get('nacos_config.jmcomic.timeout', 5.0);
    }

    private function checkNode($nodes, $timeout): array
    {
        $result = [];
        foreach ($nodes as $node) {
            try {
                $startTime = microtime(true);
                $this->client->head("https://{$node}/favicon.ico", [
                    'allow_redirects' => false, 'timeout' => $timeout,
                ]);
                $now = microtime(true);
                $result[] = ['time' => $now - $startTime, 'node' => $node];
            } catch (GuzzleException) {
            }
        }
        return $result;
    }
}
