<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */

namespace App\Jmcomic;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Hyperf\Contract\ConfigInterface;
use Hyperf\Di\Annotation\Inject;
use Hyperf\Guzzle\ClientFactory;
use Psr\Http\Message\StreamInterface;

class CDNTesting
{

    #[Inject]
    protected ConfigInterface $config;

    private Client $client;

    public function __construct(ClientFactory $clientFactory)
    {
        $this->client = $clientFactory->create();
    }

    /**
     * @throws \Exception
     */
    public function test(string $path): StreamInterface
    {
        $nodes = $this->getNodes();
        $stream = null;
        foreach ($nodes as $node) {
            $url = "https://{$node}/{$path}";
            try {
                $response = $this->client->get($url, [
                    'allow_redirects' => false, 'timeout' => 5.0, 'stream' => true,
                ]);
                $stream = $response->getBody();
                break;
            } catch (GuzzleException $e) {
                continue;
            }
        }
        if (is_null($stream)) {
            throw new \Exception('no available node');
        }
        return $stream;
    }

    private function getNodes(): array
    {
        return $this->config->get('nacos_config.jmcomic.cdn_nodes');
    }
}
