<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */
namespace App\Jmcomic;

use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\Exception\GuzzleException;
use JsonException;
use Psr\Http\Message\StreamInterface;

class Client
{
    private HttpClient $client;

    public function __construct(HttpClient $client)
    {
        $this->client = $client;
    }

    /**
     * @throws GuzzleException
     * @throws JsonException
     */
    public function filter(int $page, string $category): array
    {
        $response = $this->client->get('/categories/filter?c=hanman', [
            'proxy' => 'socks5://127.0.0.1:51837',
            'query' => [
                'page' => $page,
                'c' => $category,
            ],
        ]);
        return json_decode($response->getBody()->getContents(), true, 512, JSON_THROW_ON_ERROR);
    }

    /**
     * @throws GuzzleException
     * @throws JsonException
     */
    public function album(mixed $id)
    {
        $response = $this->client->get('album', [
            'proxy' => 'socks5://127.0.0.1:51837',
            'query' => [
                'id' => $id,
            ],
        ]);
        return json_decode($response->getBody()->getContents(), true, 512, JSON_THROW_ON_ERROR);
    }

    public function albumCoverStream($id): StreamInterface
    {
        $url = "https://cdn-msp.jmcomic2.moe/media/albums/{$id}.jpg";
        $response = $this->client->get($url, [
            'proxy' => 'socks5://127.0.0.1:51837',
            'stream' => true,
        ]);
        return $response->getBody();
    }

    /**
     * @param mixed $id
     * @return mixed
     * @throws GuzzleException
     * @throws JsonException
     */
    public function chapter(mixed $id)
    {
        $response = $this->client->get('chapter', [
            'proxy' => 'socks5://127.0.0.1:51837',
            'query' => [
                'id' => $id,
            ],
        ]);
        return json_decode($response->getBody()->getContents(), true, 512, JSON_THROW_ON_ERROR);
    }

    public function chapterPhotoStream($id, $idx): StreamInterface
    {
        $url = "https://cdn-msp.jmcomic2.moe/media/photos/{$id}/{$idx}.webp";
        $response = $this->client->get($url, [
            'proxy' => 'socks5://127.0.0.1:51837',
            'stream' => true,
        ]);
        return $response->getBody();
    }

    /**
     * @throws GuzzleException
     */
    public function albumWebInfo($id): array
    {
        $url = "https://jmcomic2.moe/album/{$id}";
        $response = $this->client->get($url, [
            'proxy' => 'socks5://127.0.0.1:51837',
        ]);
        $contents = $response->getBody()->getContents();
        $dom = new \DOMDocument();
        $dom->loadHTML($contents);
        $xpath = new \DOMXPath($dom);
        /**
         * @var \DOMNode[] $elements
         */
        $elements = $xpath->query("//div[@id='episode-block']//a[@data-album]//li");
        $chapterList = [];
        foreach ($elements as $element) {
            $chapterId = $element->parentNode->attributes->getNamedItem('data-album')->nodeValue;
            $chapterList[] = array_merge($this->matchChapter($element->textContent), ['id' => (int) $chapterId]);
        }
        return array_merge(['chapterList' => $chapterList], $this->matchDate($contents));
    }

    private function matchDate(string $contents): array
    {
        $pattern = '/上架日期 : (?<date>\d{4}-\d{2}-\d{2})/';
        preg_match($pattern, $contents, $matches);
        $date = $matches['date'];
        $pattern = '/更新日期 : (?<date>\d{4}-\d{2}-\d{2})/';
        preg_match($pattern, $contents, $matches);
        $updateDate = $matches['date'];
        return ['published' => $date, 'updateDate' => $updateDate];
    }

    private function matchChapter(string $contents): array
    {
        $contents = str_replace("最新\n", '', $contents);
        $lines = explode("\n", $contents);
        return ['chapterName' => ltrim($lines[2]), 'published' => $lines[3]];
    }
}
