<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */
namespace App\Jmcomic;

use Exception;
use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\HandlerStack;
use Hyperf\Contract\ConfigInterface;
use Hyperf\Di\Annotation\Inject;
use Hyperf\Guzzle\CoroutineHandler;

class ClientFactory
{
    #[Inject]
    protected APINodeObtain $nodeObtain;

    #[Inject]
    protected ConfigInterface $config;

    private string $node;

    /**
     * @throws Exception
     */
    public function create(): Client
    {
        if (empty($this->node)) {
            $this->node = $this->nodeObtain->get();
        }
        $stack = HandlerStack::create(new CoroutineHandler());
        $stack->push(Middleware::decrypt($this->getParam(), $this->getKey(), $this->getPaths()));
        $stack->push(Middleware::retry());

        return new Client(new HttpClient([
            'base_uri' => sprintf('https://%s/', $this->node),
            'handler' => $stack,
            'swoole' => [
                'timeout' => 10,
                'socket_buffer_size' => 1024 * 1024 * 2,
            ],
        ]));
    }

    private function getPaths() : array
    {
        return $this->config->get('nacos_config.jmcomic.paths');
    }

    private function getKey(): string
    {
        return $this->config->get('nacos_config.jmcomic.key');
    }

    private function getParam(): string
    {
        return $this->config->get('nacos_config.jmcomic.param');
    }
}
