<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */
namespace App\Jmcomic;

use GuzzleHttp\Promise\FulfilledPromise;
use GuzzleHttp\Psr7\Utils;
use GuzzleHttp\RetryMiddleware;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;

class Middleware
{
    public static function decrypt(string $params,string $key,array $paths): \Closure
    {
        return function (callable $handler) use ($params, $key, $paths) {
            return function (RequestInterface $request, array $options) use ($params, $key, $paths, $handler) {
                if (! in_array($request->getUri()->getPath(), $paths)) {
                    return $handler($request, $options);
                }
                $request = $request->withHeader('tokenparam', $params);
                /**
                 * @var FulfilledPromise $promise
                 */
                $promise = $handler($request, $options);
                return $promise->then(
                    function (ResponseInterface $response) use ($key) {
                        $body = $response->getBody();
                        $content = $body->getContents();
                        $payload = json_decode($content, true, flags: JSON_THROW_ON_ERROR);
                        if ($payload['code'] !== 200) {
                            throw new \Exception(sprintf('Request failed: %s', $payload['errorMsg']));
                        }
                        $data = openssl_decrypt($payload['data'], 'aes-256-ecb', $key);
                        $stream = Utils::streamFor($data);
                        return $response->withBody($stream);
                    }
                );
            };
        };
    }

    public static function retry(): callable
    {
        return static function (callable $handler): RetryMiddleware {
            return new RetryMiddleware(static function ($retries, RequestInterface $request, ResponseInterface $response = null) {
                if ($response && $response->getStatusCode() >= 200 && $response->getStatusCode() < 400) {
                    return false;
                }
                return true;
            }, $handler, static function ($retries) {
                return 1000 * $retries;
            });
        };
    }
}
