<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */
namespace App\Transfer;

use App\Model\ImageCenter;
use Hyperf\Di\Annotation\Inject;
use Hyperf\Nsq\Nsq;

class ImageCenterDistributor
{
    public const SOURCE_TYPE_ALBUM_THUMB = 'ALBUM_THUMB';

    public const SOURCE_TYPE_CHAPTER_IMAGE = 'CHAPTER_IMAGE';

    #[Inject]
    protected Nsq $nsq;

    /**
     * @param mixed $payload
     * @throws \Throwable
     */
    public function distribute($payload): string
    {
        $center = new ImageCenter();
        $center->source_src = $payload['source_src'];
        $center->source_type = $payload['source_type'];
        $center->unique_id = $payload['unique_id'] ?? self::generateUniqueId();
        $center->from_album_id = $payload['from_album_id'] ?? 0;
        $center->from_chapter_id = $payload['from_chapter_id'] ?? 0;
        $center->save();
        // 投递到队列里面
        // $this->nsq->publish('image_transfer', json_encode($center->getAttributes()));
        return $center->unique_id;
    }

    /**
     * @throws \Exception
     */
    public static function generateUniqueId(): string
    {
        return sha1(random_bytes(32));
    }
}
